<?php

namespace KnitPay\Extensions\BooklyPro;

use Pronamic\WordPress\Money\Money;
use Pronamic\WordPress\Money\TaxedMoney;
use Pronamic\WordPress\Money\Parser as MoneyParser;
use Pronamic\WordPress\Pay\Payments\PaymentLines;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use Pronamic\WordPress\Pay\Payments\Item;
use Pronamic\WordPress\Pay\Payments\Items;
use Pronamic\WordPress\Pay\Payments\PaymentDataInterface as Pronamic_Pay_PaymentDataInterface;
use Bookly\Lib\UserBookingData;
use Bookly\Lib\Entities\Payment as BooklyPayment;

/**
 * Title: Bookly Pro extension
 * Description:
 * Copyright: 2020 Knit Pay
 * Company: Knit Pay
 *
 * @author  knitpay
 * @since   3.4
 */
class PaymentData extends Pay_PaymentData {
	/**
	 * The user data
	 *
	 * @var UserBookingData
	 */
	private $userData;

	/**
	 * Form ID
	 *
	 * @var string
	 */
	private $form_id;

	/**
	 * Constructs and initializes an LearnPress payment data object.
	 *
	 *   @param string          $form_id     form id.
	 * @param   UserBookingData $userData   Instance LLMS_Order for the order being processed
	 */
	public function __construct( $form_id, UserBookingData $userData, BooklyPayment $bookly_payment ) {
		$this->userData       = $userData;
		$this->form_id        = $form_id;
		$this->cart_info      = $userData->cart->getInfo( 'knit_pay' );
		$this->bookly_payment = $bookly_payment;

		parent::__construct();
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'bookly-pro';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return $this->bookly_payment->getId();
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		return $this->userData->cart->getItemsTitle( 128, false );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
		$description = get_option( 'bookly_knit_pay_payment_description' );

		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{form_id}'      => $this->form_id,
			'{service_name}' => $this->userData->cart->getItemsTitle( 128, false ),
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return $this->bookly_payment->getId();
	}

	public function get_items() {

	}

	public function get_amount() {
		$amount = new TaxedMoney( $this->cart_info->getGatewayAmount(), $this->get_currency_alphabetic_code() );
		return $amount;

		/*
		 $total = 0;
		if ( $items = LP()->get_cart()->get_items() ) {
			foreach ( $items as $item ) {
			   $total += $item['total'];
			}
		}
		$amount = new TaxedMoney($total,$this->get_currency_alphabetic_code());
		return $amount; */
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
		return get_option( 'bookly_pmt_currency' );
	}

	/**
	 * Get email.
	 *
	 * @return string
	 */
	public function get_email() {
		return esc_attr( $this->userData->getEmail() );
	}

	/**
	 * Get first name.
	 *
	 * @return string
	 */
	public function get_first_name() {
		return $this->userData->getFirstName();
	}

	/**
	 * Get last name.
	 *
	 * @return string
	 */
	public function get_last_name() {
		return $this->userData->getLastName();
	}

	/**
	 * Get address.
	 *
	 * @return null|string
	 */
	public function get_address() {
		return $this->userData->getAdditionalAddress();
	}

	/**
	 * Get city.
	 *
	 * @return null|string
	 */
	public function get_city() {
		return $this->userData->getCity();
	}

	/**
	 * Get ZIP.
	 *
	 * @return null|string
	 */
	public function get_zip() {
		return $this->userData->getPostcode();
	}

	/**
	 * Get telephone number.
	 *
	 * @return null|string
	 */
	public function get_telephone_number() {
		return $this->userData->getPhone();
	}
}
